
(require "iterate")
(require "general")
(require "conditions" "conditions.lisp")
(require "mcp-read")
(require "tk")

(provide "mud-client")

(defvar *mud-name* "LocalMOO")    ; I use LocalMOO for my local experiments.
(defvar *mud-character* nil) 
(defvar *mud-role* 'avatar) 
(defvar *mud-log* nil) 

;; (Mud Host Socket (UserId Pwd Role)*)
(defvar *muds*
  '(("LambdaMOO"
     "lambda.parc.xerox.com" 8888)
    ("Druid"
     "pueblo.xerox.com" 4321
     ("mikeS" "napuxit" agent))
    ("Pueblo"
     "pueblo.xerox.com" 7777
     ("komodo" "padedon" avatar) ("MikeS" "leretyr" agent))
    ("CAETI"
     "caesar.uta.edu" 7777
     ("komodo" "padedon" avatar) ("MikeS" "leretyr" agent) ("Amy" "wolanud" student))
    ("LocalMOO"
     "localhost" 7777
     ("wizard" "" agent))))

(defvar *mud-connect* "connect ~a ~a")
(defvar *mud-disconnect* "@quit")

(eval-when (compile)
 (load-base "tk"))

(defun telnet (host socket logfile) (tk::telnet host logfile socket))

(defun send-telnet (string) (tk::send-telnet string))

(defvar *mud-count* 0)
(defvar *from-mud-root* "/tmp/mud-out.~d")
(defvar *from-mud-file* nil)

(defvar *mud-session-root* "mud.log.~d")
(defvar *mud-session-log* nil)

(defparameter *mud-retrys* 20)
(defparameter *mud-sleep* .25)

(defun mud-user-pwd (who mud)
  (cadr (assoc who (cdddr mud) :test #'equal)))

(defun mud-role-id (r mud)
  (mud-role-id2 r (cdddr mud)))

(defun mud-role-id2 (r characters)
  (cond ((not (consp characters)) nil)
	((not (consp (car characters))) nil)
	((equal r (caddr (car characters))) (car (car characters)))
	(t (mud-role-id2 r (cdr characters)))))

(defun connect-to-mud (&optional (where *mud-name*) (who *mud-character*) (role *mud-role*) pwd)
  (let ((mud (assoc where *muds* :test #'string-equal)))
    (cond ((and who pwd))
	  (who  (setq pwd (mud-user-pwd who mud)))
	  (role (setq who (mud-role-id role mud))
		(setq pwd (mud-user-pwd who mud)))
	  (t (setq who (car (cdddr mud)))
	     (setq pwd (mud-user-pwd who mud))))
    (cond ((or (null who) (null pwd)) 
	   (format t "~%~%*** No user id (or pwd) in mud, ~a, for [who: ~a, role: ~a]~%"
		   where who role)
	   nil)
	  ((and (boundp 'tk::telnet-stream)
		(streamp tk::telnet-stream)
		(string-equal where *mud-name*)
		(open-mud where))
	   t)
	  (t (setq *mud-count* (+ *mud-count* 1))
	     (setq *from-mud-file* (format nil *from-mud-root* *mud-count*))
	     (if (null mud)
		 (error (format nil "Unknown MUD: ~a" where))
		 (let ((host (cadr mud))
		       (socket (caddr mud))
		       (retry 0)
		       found)
		   (telnet host socket *from-mud-file*)
		   (iterate while (and (not found) (< retry *mud-retrys*))
			    do (progn (sleep *mud-sleep*)
				      (setq retry (+ retry 1))
				      (if (open *from-mud-file* :direction :probe)
					  (progn (setq found t)
						 (setq *mcp-input*
						       (open *from-mud-file* :direction :input)))
					  nil)))
		   (if found
		       (progn (mud-initialize-log)
			      (send-to-mud (format nil *mud-connect* who pwd))
			      t)
		       nil)))))))

(defun exit-mud ()
  (send-telnet *mud-disconnect*)
  (sleep *mud-sleep*)
  (close *mud-log*))

(defvar *mud-logging* nil)

(defun mud-initialize-log ()
  (tk::clear-both-output-windows)
  (if *mud-logging*
      (progn (setq *mud-session-log* (format nil *mud-session-root* *mud-count*))
	     (setq *mud-log* (open *mud-session-log* :direction :output)))))

(defun cmd-to-mud (string)
  (if (stringp string) (tk::insert-mcp string))
  (if (streamp *mud-log*)
      (format *mud-log* (format nil "~a~%" string)))
  (send-telnet (format nil "###~a~%" string)))

(defun send-to-mud (string)
  (if (stringp string) (tk::insert-output string))
  (if (streamp *mud-log*)
      (format *mud-log* (format nil "~a~%" string)))
  (send-telnet (format nil "~a~%" string)))

