
;; ------------------------------------------------------------
;; William R. Bevier and Michael K. Smith.
;; Copyright 1994, Computational Logic, Inc. All Rights Reserved
;; ------------------------------------------------------------

(require "general")
(provide "system")

;; ------------------------------------------------------------
;; Changes for Version3 : CAETI
;
;; Simplified.
;; Merged conditions and vars
;; Merged defined events, actions and conditions.
;; Deleted history.
;; Deleted prim-conditions, merged with -vars.

;; ------------------------------------------------------------

;; ------------------------------------------------------------
;; Structure of the System Under Description (SYSTEM)
;; ------------------------------------------------------------

(eval-when (load compile eval)

 (defstruct system
   ;; Dynamic fields of a SYSTEM
   (time 0)				; current time
   configuration			; set of active (current) state names
   alist				; var -> value
   schedule				; a list of ((time . event*)*)
					;  time = 0 indicates immediate
					;  monotonically increasing in time.
   ;; Static fields of a SYSTEM
   root                                 ; name of root node
   nodes				; set of state structures
   arcs					; set of arc structures
   prim-events				; a list of patterns whose cars are symbolp.
   )

 (defmacro update-system (x &rest rest)
   `(make-system ,@(update-fields 'system
				  x
				  '(time configuration alist 
					 schedule 
					 root nodes arcs
					 prim-events)
				  rest)))
 )

;; (actionp    form prim-evs states vars)
;; (eventp     form prim-evs states vars)
;; (conditionp form states vars)

(defun systemp (s)
  (let* ((root (system-root s))
	 ;;(conf (system-configuration s))
	 (vars (system-alist s))
	 ;;(schedule (system-schedule s))
	 (nodes (system-nodes s))
	 (names (node-names nodes))
	 (arcs  (system-arcs s))
	 (prim-events (system-prim-events s)))
    (and (static-check (full-node-listp nodes root) "~%SYSTEMP: node failed.")
	 (static-check (full-arc-listp arcs nodes) "~%SYSTEMP: arc failed.")
	 (static-check (actions-and-triggers-ok arcs prim-events names vars)
		      "~%SYSTEMP: action or trigger failed.")
	 (static-check (member (system-root s) names) "~%SYSTEMP: Root not a node name."))))


(defun compute-state-tree (system)
  (compute-tree (system-root system) (system-nodes system)))

(defun compute-childtree (kids nodes)
  (if (null kids)
      nil
      (cons (compute-tree (car kids) nodes)
	    (compute-childtree (cdr kids) nodes))))    

(defun compute-tree (nm nodes)
  (let ((type (ntype nm nodes)))
    (if (or (eq type 'and) (eq type 'or))
	(cons nm
	      (cons type
		    (compute-childtree (nchildren nm nodes) nodes)))
	nm)))
