(in-package "TK")

(provide "tk")

(defvar *tk-color-alist*
  '((black    "#000")
    (darkblue "#00f")
    (green    "#0f0")
    (aquamarine "#0ff")
    (red      "#f00")
    (purple   "#f0f")
    (yellow   "#ff0")
    (white    "#fff")))

(defvar *default-highlight-color* "#0ff")

;; Widgets

(defvar *widget-count* 1)

(defun clear-text-widget (x)
  (funcall x :delete "1.0" "end")
  ;(funcall x :configure :height 1)
  )

(defun create-text-widget (parent)
  (let ((tt (text (conc parent '.text) :height 1)))
    (pack tt :side "bottom")
    tt))

(defun old-insert-line-in-text-widget (x string)
  (let ((h (nth 4 (funcall x :configure :height :return 'list-strings))))
    (if (and h (not (string= h "")))
	(setq h (read-from-string h)))
    (funcall x :configure :height (+ h 1))
    (funcall x :insert "end" string)
    (funcall x :insert "end" "
")))

(defun insert-line-in-text-widget (x string)
  (funcall x :insert "end" string)
  (funcall x :yview "end - 10 lines"))

(defvar *mud-control-panel* "Mud Client")
(defvar *mud-output* nil)
(defvar *mud-msgs* nil)

(defun clear-both-output-windows ()
  (clear-text-widget *mud-output*)
  (clear-text-widget *mud-msgs*))

(defun insert-output (string)
  (insert-line-in-text-widget *mud-output* string))

(defun insert-mcp (string)
  (insert-line-in-text-widget *mud-msgs* string))

(eval-when (compile load eval)
 (defmacro tk-exists (x)
  `(and (boundp ',x)
	,x
	(string= (winfo :exists ,x) "1"))))

(defvar *tk-topmud* '.mud)

(defun mk-mud-panel () 
  (let ((w *tk-topmud*)
	wmudf wcomf tmud tcom smud scom
	ws saylab saytxt
	wc comlab comtxt
	wb b1 b2 b3 b4 b5 b6 b7 b8)
    (if (not *tk-connection*) (tkconnect))
    (if (not (tk-exists w))   (toplevel w))
    ;; (dpos w)
    (wm :title w "Mud Client")
    (wm :iconname w "Mud")
    (wm :geometry w "-500+40")

    (setq wmudf (conc w '.mudf))
    (setq wcomf (conc w '.comf))
    (setq tmud (conc wmudf '.txt))
    (setq tcom (conc wcomf '.txt))
    (setq smud (conc wmudf '.scroll))
    (setq scom (conc wcomf '.scroll))
    (setq wb (conc w '.b))
    (setq ws (conc w '.say))
    (setq wc (conc w '.com))

    (frame wmudf :height 18 :bd 2 :relief "raised")
    (scrollbar smud :command (tk-conc tmud " yview") :relief "raised")
    (text tmud :yscroll (tk-conc smud " set"))

    (frame wcomf :height 18 :bd 2 :relief "raised")
    (scrollbar scom :command (tk-conc tcom " yview") :relief "raised")
    (text tcom :yscroll (tk-conc scom " set")) ;; :wrap "none"

    (frame wb :height 2)	 ;buttons
    (frame wc :height 2)	 ; to cmd
    (frame ws :height 2)	 ; to Say

    (setq *mud-output* tmud)
    (setq *mud-msgs* tcom)

    (pack smud :side "right" :fill "y")
    (pack tmud :side "left" :pady 2 :fill "x")

    (pack scom :side "right" :fill "y")
    (pack tcom :side "left" :pady 2 :fill "x")

    (pack wmudf wcomf ws wc wb :side "top" :fill "x" :pady 1)

    (setq saylab (conc ws '.label))
    (setq saytxt (conc ws '.text))
    (label saylab :text "Say " :width 12 :relief "raised")
    (entry saytxt :textvariable "sayvalue" :relief "raised" :width 80)
    (bind  saytxt "<Return>" "writemoo \"say $sayvalue\"")
    (pack saylab saytxt :side "left" :fill "x")

    (make-editing-entry saytxt)

    (setq comlab (conc wc '.comlabel))
    (setq comtxt (conc wc '.comtext))
    (label comlab :text "Command " :width 12 :relief "raised")
    (entry comtxt :textvariable "comvalue" :relief "raised" :width 80)

    (make-editing-entry comtxt)
    (bind comtxt "<Return>" (format nil "writemoo $comvalue
set oldvalue $comvalue
~a delete 0 end
" (string-downcase comtxt)))

    (pack comlab comtxt :side "left" :fill "x")

    (setq b1 (conc wb '.connect))
    (setq b2 (conc wb '.quit))
    (setq b3 (conc wb '.look))
    (setq b4 (conc wb '.who))
    (setq b5 (conc wb '.north))
    (setq b6 (conc wb '.south))
    (setq b7 (conc wb '.east))
    (setq b8 (conc wb '.west))

    ;;(button b1 :text "Connect" :command "writemoo \"connect\n\"" :width 12)

    (button b3 :text "Look"  :width 12 :command "writemoo \"look
\"")
    (button b4 :text "Who"   :width 12 :command "writemoo \"@who
\"")
    (button b5 :text "North" :width 12 :command "writemoo \"go north
\"")
    (button b6 :text "South" :width 12 :command "writemoo \"go south
\"")
    (button b7 :text "East"  :width 12 :command "writemoo \"go east
\"")
    (button b8 :text "West"  :width 12 :command "writemoo \"go west
\"")
    (button b2 :text "Quit"  :width 12 :command "writemoo \"@quit
\"
after 2000 { exit }
")

    (pack b3 b4 b5 b6 b7 b8 b2 :side "left" :expand "yes" :fill "x")

    (bind w "<Any-Enter>" `(focus ',w))
    (focus w)))

(defun make-editing-entry (entry)
  (let ((entry (string-downcase entry)))
    (bind  entry "<Meta-p>"    (format nil "~a insert 0 $oldvalue" entry))
    (bind  entry "<Control-k>" (format nil "set saved [string range $comvalue [~a index insert] end]
~a delete insert end" entry entry))
    (bind  entry "<Meta-k>"    (format nil "set saved [string range $comvalue 0 [~a index insert]]
~a delete 0 insert" entry entry))
    (bind  entry "<Control-a>" (format nil "~a icursor 0"  entry))
    (bind  entry "<Control-e>" (format nil "~a icursor end" entry))
    (bind  entry "<Control-b>" (format nil "set mark [expr [~a index insert]-1]
~a icursor $mark" entry entry))
    (bind  entry "<Control-f>" (format nil "set mark [expr [~a index insert]+1]
~a icursor $mark" entry entry))
    (bind  entry "<Control-w>" (format nil "set saved [~a get]" entry))
    (bind  entry "<Control-y>" (format nil "~a insert [~a index insert] $saved" entry entry))))

(defvar *tk-topstate* '.state)

(defvar *statemap* nil)

(defun mk-state (s up &optional (stacking "top"))
  (if (consp s)
      (let* ((frame (conc up '.f (car s)))
	     (label (conc frame '.l (car s)))
	     (type  (cadr s)) 
	     (next  (if (string-equal stacking "left")
			"top"
			"left"))
	     ;; (fill  (if (string-equal stacking "top") "x" "y"))
	     (relief (if (equal type 'or) "groove" "sunken"))
	     (sons  (cddr s)))
	(frame frame :relief relief :borderwidth 2)
	(label label :text (string (car s)))
	(pack label :side stacking :pady 2 :padx 2)
	(setq sons (mapcar (function (lambda (x) (mk-state x frame next)))
			   sons))
	(pack frame :pady 4 :padx 4 :fill "both")
	(setq *statemap* (cons (cons (car s) label) *statemap*))
	frame)
      (let* ((label (conc up "." s)))
	(label label :text (string s) :relief "raised")
	(pack label :side stacking :pady 2 :padx 2 :fill "both")
	(setq *statemap* (cons (cons s label) *statemap*))
	label)))

(defun highlight (state)
  (let ((s (cdr (assoc state *statemap*))))
    (if s (funcall s :configure :bg "pink"))))   

(defun lowlight (state)
  (let ((s (cdr (assoc state *statemap*))))
    (if s (funcall s :configure :bg "#ffe4c4"))))   

(defun node-update (old config)
  ;; First turn off old config, then highlight elements of configuration.
  (mapcar (function (lambda (x) (lowlight  x))) old)
  (mapcar (function (lambda (x) (highlight x))) config))

(defun mk-state-panel (states &optional (w *tk-topstate*))
  (let (chart)
    (if (not *tk-connection*) (tkconnect))
    (destroy w)
    (if (not (tk-exists w))   (toplevel w))
    ;; (dpos w)
    (setq *statemap* nil)
    (wm :title w "Robot State")
    (wm :iconname w "State")
    (wm :geometry w "+10+40")

    (setq chart (conc w '.chart))
    (frame chart)
    (pack chart :side "left" :fill "both")

    (mk-state states chart)

    (bind w "<Any-Enter>" `(focus ',w))
    (focus w)))

;;; TELNET Stuff

(defvar telnet-stream nil)
  
(defun telnet (host logfile &optional (socket ""))
  (setq telnet-stream  (tk-call "trytelnet" host socket logfile :return 'string )))

(defun send-telnet (string)
  (tk-call "writeflush" telnet-stream string)) 

(defun send-line (string)
  (tk-call "writeflush" telnet-stream string : #u"\n"))

;;; INITIALIZATION

(defun start-tk ()
  (tk::tkconnect)
  (tk-do "
proc trytelnet {host socket logfile} {
 global telnetsocket
 set telnetsocket [open [list |telnet $host $socket >& $logfile] {WRONLY NONBLOCK NOCTTY}]
 return $telnetsocket
}
proc writeflush { file string } {
  puts $file $string
  flush $file
}
proc writemoo { string } {
  global telnetsocket
  catch { puts $telnetsocket $string
          flush $telnetsocket }
}
")
  (mk-mud-panel))


